#define NOMINMAX
#include <bblean/BBApi.h>
#include <vector>
#include <map>
#ifdef LOG
#include <fstream>
#endif

enum {BBLEAN,BB4WIN,XOBLITE};
int branch=0;

class owcCommand
{
public:
	virtual ~owcCommand();
	virtual void execute() = 0;
};

owcCommand::~owcCommand() {}

class owcBroam:public owcCommand
{
public:
	owcBroam(const char *);
	~owcBroam();
	virtual void execute();
private:
	char *data;
};

owcBroam::owcBroam(const char *input)
{
	data = new char[strlen(input)+1];
	strcpy(data,input);
}

owcBroam::~owcBroam()
{
	delete [] data;
}

void owcBroam::execute()
{
	SendMessage(GetBBWnd(),BB_BROADCAST,0,(LPARAM)(data));
}

class owcMenu:public owcCommand
{
public:
	owcMenu(const char *);
	~owcMenu();
	virtual void execute();
private:
	char *data;
};

owcMenu::owcMenu(const char *input)
{
	data = new char[strlen(input)+1];
	strcpy(data,input);
}

owcMenu::~owcMenu()
{
	delete [] data;
}

void owcMenu::execute()
{
	char path[MAX_PATH];
	path[0]=0;
	if (branch==XOBLITE) 
	{
		strcpy(path,"@xoblite Menu SetRCPath $Blackbox$\\");
		strcat(path,data);
		SendMessage(GetBBWnd(),BB_BROADCAST,0,(LPARAM)(path));
	}
	else
		menuPath(data);
}

char rcpath[MAX_PATH];
const char *findHelp;
HINSTANCE hInstance;
#ifdef LOG
std::ofstream logFile;
#endif
static HANDLE   hProcess = NULL;
static HWND     hPluginWnd = NULL;
static int      bb_messages[] = {BB_BROADCAST, BB_RECONFIGURE, BB_DESKTOPINFO, BB_WORKSPACE, BB_WINDOWMINIMIZE, BB_BRINGTOFRONT, 0};
DesktopInfo currentDeskInfo,oldDeskInfo;
void ReadRCSettings(void);

LPCSTR pluginInfo(int x)
{
	switch (x)
	{
		case PLUGIN_NAME:			return "bbOnWorkspaceChange";
		case PLUGIN_VERSION:		return "0.0.3";
		case PLUGIN_AUTHOR:			return "Carsomyr";
		case PLUGIN_RELEASEDATE:	return "2006/02/10";
		case PLUGIN_LINK:			return "None";
		case PLUGIN_EMAIL:			return "s23120712@tuks.co.za";
		default:					return "bbOnWorkspaceChange 0.0.3";
	}
}

//---------------------------------------------------------------------------------
class strCmp {
    public:
    bool operator()( const char* s1, const char* s2 ) const {
      return strnicmp( s1, s2 ,s2==findHelp?strlen(s1):strlen(s2)) < 0;
    }
  };

std::map<const char *,std::vector<owcCommand *>,strCmp> triggerMap;
typedef std::map<const char *,std::vector<owcCommand *>,strCmp>::iterator TriggerIterator;
typedef std::vector<owcCommand *>::iterator ActionIterator;
//---------------------------------------------------------------------------------

void executeAction(DesktopInfo &workspace, const char* verb)
{
	char buf[1000],buf2[1000];
	findHelp=buf;
	TriggerIterator found;
	strcpy(buf,verb);
	strcat(buf,workspace.name);
	if ((found=triggerMap.find(buf))!=triggerMap.end())
		for(ActionIterator i = (found->second).begin();i!=(found->second).end();++i)
			(*i)->execute();
	strcpy(buf,verb);
	itoa(workspace.number,buf2,10);
	strcat(buf,buf2);
	if ((found=triggerMap.find(buf))!=triggerMap.end())
		for(ActionIterator i = (found->second).begin();i!=(found->second).end();++i)
			(*i)->execute();
}

LRESULT CALLBACK PluginProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(message)
    {
	case BB_BROADCAST:
		break;
	case BB_RECONFIGURE:
		ReadRCSettings();
		break;
	case BB_DESKTOPINFO:
	{
		currentDeskInfo=*((DesktopInfo*)lParam);
		if (currentDeskInfo.isCurrent)
		{
			if (currentDeskInfo.number!=oldDeskInfo.number)
			{
				#ifdef LOG
				logFile<<"Actions "<<currentDeskInfo.name<<" "<<oldDeskInfo.name<<std::endl;
				#endif
				executeAction(oldDeskInfo,"leave.");
				executeAction(currentDeskInfo,"enter.");
			}
			#ifdef LOG
			else
				logFile<<"NoAction "<<currentDeskInfo.name<<" "<<oldDeskInfo.name<<std::endl;
			#endif
			oldDeskInfo=currentDeskInfo;
		}
		break;
	}
	case BB_WORKSPACE:
	{
		#ifdef LOG
		logFile<<"Workspace "<<wParam<<" "<<lParam<<std::endl;
		#endif
		if (wParam!=BBWS_MINIMIZEALL) SendMessage(GetBBWnd(),BB_LISTDESKTOPS,WPARAM(hwnd),0);
		break;
	}
	case BB_WINDOWMINIMIZE:
	{
		#ifdef LOG
		logFile<<"Minimize"<<std::endl;
		#endif
		SendMessage(GetBBWnd(),BB_LISTDESKTOPS,WPARAM(hwnd),0);
		break;
	}
	case BB_BRINGTOFRONT:
	{
		#ifdef LOG
		logFile<<"BringToFront"<<std::endl;
		#endif
		SendMessage(GetBBWnd(),BB_LISTDESKTOPS,WPARAM(hwnd),0);
		break;
	}
	default:return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0;
}

//---------------------------------------------------------------------------------

int beginPluginEx(HINSTANCE h_instance, HWND hSlit)
{
    return beginPlugin(h_instance);
}

int beginSlitPlugin(HINSTANCE h_instance, HWND hSlit)
{
    return beginPlugin(h_instance);
}

int beginPlugin(HINSTANCE h_instance)
{
	#ifdef LOG
	logFile.open("log.txt");
	#endif
    hInstance=h_instance;
    if (!strnicmp(GetBBVersion(),"bblean",6))
        branch=BBLEAN;
    else if (!strnicmp(GetBBVersion(),"0.",2))
        branch=BB4WIN;
    else
        branch=XOBLITE;
    ReadRCSettings();
    WNDCLASS wc;
    ZeroMemory(&wc, sizeof(wc));
    wc.lpfnWndProc = PluginProc;
    wc.hInstance = h_instance;
    wc.lpszClassName = pluginInfo(PLUGIN_NAME);
    RegisterClass(&wc);

    hPluginWnd = CreateWindowEx(
        WS_EX_TOOLWINDOW,
        pluginInfo(PLUGIN_NAME),
        NULL,
        WS_POPUP|WS_ICONIC,
        -4, -4, 0, 0,
        HWND_MESSAGE,
        NULL,
        h_instance,
        NULL
    );
    hProcess = GetCurrentProcess();
    SendMessage(GetBBWnd(), BB_REGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
    strcpy(oldDeskInfo.name,"");
	oldDeskInfo.number=-1;
	SendMessage(GetBBWnd(),BB_LISTDESKTOPS,WPARAM(hPluginWnd),0);
    return 0;
}

//---------------------------------------------------------------------------------

void endPlugin(HINSTANCE h_instance)
{
	#ifdef LOG
	logFile.close();
	#endif
	for (TriggerIterator i = triggerMap.begin(); i != triggerMap.end(); ++i)
	{
		for(ActionIterator j = (*i).second.begin();j != (*i).second.end(); ++j)
			delete *j;
	}
	triggerMap.clear();
    DestroyWindow(hPluginWnd);
    SendMessage(GetBBWnd(), BB_UNREGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
    UnregisterClass(pluginInfo(PLUGIN_NAME), h_instance);
}

//---------------------------------------------------------------------------------
void ReadRCSettings(void)
{
    for (TriggerIterator i = triggerMap.begin(); i != triggerMap.end(); ++i)
	{
		for(ActionIterator j = (*i).second.begin();j != (*i).second.end(); ++j)
			delete *j;
	}
	triggerMap.clear();
    char name[MAX_PATH];
	int i = 0;
	do
	{
		// First and third, we look for the config file
		// in the same folder as the plugin...
		HINSTANCE hInst = hInstance;
		//Allow us to get the name of dll used
		GetModuleFileName(hInst, rcpath, sizeof(rcpath));
		char *plugin_name = strrchr(rcpath, '\\');
		plugin_name++;
		char *extension_start = strrchr(plugin_name, '.');
		extension_start++;
		extension_start[0]='r';
        extension_start[1]='c';
        extension_start[2]=0;
        strcpy(name,plugin_name);
		// second we check the blackbox directory
		if (1 == i) hInst = NULL;

		GetModuleFileName(hInst, rcpath, sizeof(rcpath));
		char *file_name_start = strrchr(rcpath, '\\');
                
		if (file_name_start) ++file_name_start;
		else file_name_start = strchr(rcpath, 0);
		strcpy(file_name_start, name);
	} 
	while (++i < 3 && false == FileExists(rcpath));
	// If a config file was found we read the plugin settings from it...
	// ...if not, the ReadXXX functions give us just the defaults.
    
    long int readIndex = 0;
    const char *data = NULL;
    do
    {
        data = ReadValue(rcpath,"bbOWC.workspace:",&readIndex);
        if (branch==XOBLITE) ++readIndex;
        if (data)
        {
            char *trigger = new char[strlen(data)+7];
            std::vector<owcCommand *> blank;
            strcpy(trigger,"enter.");
            strcat(trigger,data);
            triggerMap[trigger]=blank;
            trigger = new char[strlen(data)+7];
            strcpy(trigger,"leave.");
            strcat(trigger,data);
            triggerMap[trigger]=blank;
        }
    }
    while(data);
	//Read out broam items
    for (TriggerIterator i = triggerMap.begin(); i!=triggerMap.end(); ++i)
    {
        char key[strlen("bbOWC.")+strlen((*i).first)+strlen(".broam:")+1];
        strcpy(key,"bbOWC.");
        strcat(key,(*i).first);
        strcat(key,".broam:");
        readIndex=0;
        do
        {
            data = ReadValue(rcpath,key,&readIndex);
            if (branch==XOBLITE) ++readIndex;
            if (data)
            {
                findHelp=(*i).first;
                triggerMap[(*i).first].push_back(new owcBroam(data));
            }
        }
        while(data);
    }
	//Read out menu items
    for (TriggerIterator i = triggerMap.begin(); i!=triggerMap.end(); ++i)
    {
        char key[strlen("bbOWC.")+strlen((*i).first)+strlen(".menu:")+1];
        strcpy(key,"bbOWC.");
        strcat(key,(*i).first);
        strcat(key,".menu:");
        readIndex=0;
        do
        {
            data = ReadValue(rcpath,key,&readIndex);
            if (branch==XOBLITE) ++readIndex;
            if (data)
            {
                findHelp=(*i).first;
                triggerMap[(*i).first].push_back(new owcMenu(data));
            }
        }
        while(data);
    }
}
